﻿using KelownaSoftware.Application.Common.Exceptions;
using KelownaSoftware.Domain.AggregateRoots.Users;
using KelownaSoftware.Domain.Common.Exceptions;
using KelownaSoftware.Domain.Common.Validation;
using KelownaSoftware.Platform.Common.Extensions;
using MediatR;
using Microsoft.AspNetCore.Identity;

namespace KelownaSoftware.Application.AggregateRoots.Authentication.ConfirmEmailAddress
{
    public class ConfirmEmailAddressCommand : IRequest<ConfirmEmailAddressResult>
    {
        public string Token { get; set; }
        public string EncodedEmail { get; set; }

        public class ConfirmEmailAddressCommandHandler : IRequestHandler<ConfirmEmailAddressCommand, ConfirmEmailAddressResult>
        {
            private readonly UserManager<User> _userManager;

            public ConfirmEmailAddressCommandHandler(UserManager<User> userManager)
            {
                _userManager = userManager;
            }

            public async Task<ConfirmEmailAddressResult> Handle(ConfirmEmailAddressCommand request, CancellationToken cancellationToken)
            {
                var emailAddress = DecodeEmail(request.EncodedEmail);
                var user = await GetUserAsync(emailAddress);

                CheckIfEmailAlreadyConfirmed(user);

                var decodedToken = DecodeToken(request.Token);
                var confirmResult = await ConfirmEmailAddressAsync(user, decodedToken);

                var result = GenerateresultFromIdentityResult(confirmResult, emailAddress);

                return result;

            }

            public string DecodeEmail(string encodedEmail)
            {
                var decodedEmail = encodedEmail.FromBase64String();
                return decodedEmail;
            }

            public async Task<User> GetUserAsync(string emailAddress)
            {
                var user = await _userManager.FindByNameAsync(emailAddress);

                _ = user ?? throw new EntityNotFoundException(nameof(User), emailAddress);

                return user;
            }

            public void CheckIfEmailAlreadyConfirmed(User user)
            {
                if (user.EmailConfirmed)
                {
                    throw new BaseException("Unable to confirm the Email Address, it has already been confirmed.");
                }
            }

            public string DecodeToken(string encodedToken)
            {
                var decodedToken = Uri.UnescapeDataString(encodedToken);
                return decodedToken;
            }

            public async Task<IdentityResult> ConfirmEmailAddressAsync(User user, string token)
            {
                var confirmResult = await _userManager.ConfirmEmailAsync(user, token);
                return confirmResult;
            }

            public ConfirmEmailAddressResult GenerateresultFromIdentityResult(IdentityResult identityResult, string emailAddress)
            {
                var result = new ConfirmEmailAddressResult(identityResult.Succeeded, emailAddress);

                if (identityResult.Succeeded == false)
                {
                    result.AddValidationError(new ValidationError() { });
                    result.Error = identityResult.Errors.FirstOrDefault().Description ?? throw new Exception("An unknown error ocurred");
                }

                return result;

            }
        }
    }
}
