﻿using KelownaSoftware.Application.Common.Interfaces;
using KelownaSoftware.Application.Common.Notifications.Email;
using KelownaSoftware.Application.Common.Utilities;
using KelownaSoftware.Domain.AggregateRoots.Users;
using MediatR;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Identity;

namespace KelownaSoftware.Application.AggregateRoots.Authentication.ForgotPassword
{
    public class ForgotPasswordCommand : IRequest<ForgotPasswordResult>
    {
        public string Email { get; set; }

        public class ForgotPasswordCommandHandler : IRequestHandler<ForgotPasswordCommand, ForgotPasswordResult>
        {
            private readonly UserManager<User> _userManager;
            private readonly IEmailService _emailService;
            private readonly IHttpContextAccessor _httpContextAccessor;

            public ForgotPasswordCommandHandler(UserManager<User> userManager, IHttpContextAccessor httpContextAccessor, IEmailService emailService)
            {
                _userManager = userManager;
                _emailService = emailService;
                _httpContextAccessor = httpContextAccessor;
            }

            public async Task<ForgotPasswordResult> Handle(ForgotPasswordCommand request, CancellationToken cancellationToken)
            {
                var result = new ForgotPasswordResult();

                //prevent returning a 500 error, even if the user doesn't exist or sendgrid fails isn't found
                try
                {
                    var user = await _userManager.FindByNameAsync(request.Email);

                    if (user != null)
                    {
                        var passwordResetToken = await _userManager.GeneratePasswordResetTokenAsync(user);
                        var baseUrl = HttpContextHelpers.GetBaseUrl(_httpContextAccessor);
                        var forgotPasswordEmailNotification = new ForgotPasswordEmailNotification(request.Email, passwordResetToken, baseUrl);

                        await _emailService.SendAsync(forgotPasswordEmailNotification, cancellationToken);
                    }
                }
                catch
                {
                    //we don't need to log or do anything here because:
                    // 1. The email may not have been found (user doesn't exist)
                    // 2. The email address may be invalid
                    // 3. The email may have failed to send, which is logged elsewhere
                }

                result.Success = true;

                return result;
            }
        }
    }
}
