﻿using KelownaSoftware.Application.Common.Interfaces.Services;
using KelownaSoftware.Application.Interfaces;
using KelownaSoftware.Application.Interfaces.Authentication;
using KelownaSoftware.Application.Interfaces.Authentication.Jwt;
using KelownaSoftware.Domain.AggregateRoots.Users;
using KelownaSoftware.Domain.Common.Exceptions;
using MediatR;
using Microsoft.AspNetCore.Identity;
using System.Security.Claims;

namespace KelownaSoftware.Application.AggregateRoots.Authentication.RefreshToken
{
    public class RefreshTokenCommand : IRequest<RefreshTokenResult>
    {
        public string Token { get; set; }
        public string RefreshToken { get; set; }
        public string CompanyId { get; set; }

        public class RefreshTokenCommandHandler : IRequestHandler<RefreshTokenCommand, RefreshTokenResult>
        {
            private readonly UserManager<User> _userManager;
            private readonly IJwtTokenValidatorService _jwtTokenValidator;
            private readonly IJwtConfigurationService _jwtConfig;
            private readonly IJwtService _jwtService;
            private readonly IRefreshTokenService _refreshTokenService;
            private readonly IKSDbContext _dbContext;
            private readonly IPermissionsService _appPermissionsService;


            public RefreshTokenCommandHandler(
                UserManager<User> userManager,
                IJwtTokenValidatorService jwtTokenValidator,
                IJwtService jwtService,
                IRefreshTokenService refreshTokenService,
                IJwtConfigurationService jwtConfig,
                IKSDbContext dbContext,
                IPermissionsService appPermissionsService)
            {
                _userManager = userManager;
                _jwtTokenValidator = jwtTokenValidator;
                _jwtConfig = jwtConfig;
                _jwtService = jwtService;
                _refreshTokenService = refreshTokenService;
                _dbContext = dbContext;
                _appPermissionsService = appPermissionsService;
            }

            public async Task<RefreshTokenResult> Handle(RefreshTokenCommand request, CancellationToken cancellationToken)
            {
                var response = new RefreshTokenResult(string.Empty, string.Empty);

                var cp = _jwtTokenValidator.GetPrincipalFromToken(request.Token, _jwtConfig.GetPrivateKey());

                // invalid token/signing key was passed and we can't extract user claims
                if (cp != null)
                {
                    var userId = cp.Claims.First(c => c.Type == ClaimTypes.NameIdentifier).Value;
                    var user = await _userManager.FindByIdAsync(userId);

                    if (user != null)
                    {
                        if (await _refreshTokenService.HasValidRefreshToken(user.Id, request.RefreshToken).ConfigureAwait(false))
                        {
                            var claims = await _appPermissionsService.GenerateUserClaimsAsync(user.Id, user.Email, cancellationToken);
                            var jwt = await _jwtService.GenerateEncodedToken(claims);
                            var refreshToken = _refreshTokenService.GenerateToken();

                            await _refreshTokenService.RemoveRefreshTokenAsync(request.RefreshToken, cancellationToken).ConfigureAwait(false);
                            await _refreshTokenService.AddRefreshTokenAsync(refreshToken, user.Id, "", cancellationToken).ConfigureAwait(false);

                            response = new RefreshTokenResult(jwt.Token, refreshToken);
                        }
                    }
                    else
                    {
                        throw new UnexpectedStateException("Null User on RefreshTokenCommand");
                    }
                }

                return response;
            }
        }
    }
}
