﻿using KelownaSoftware.Application.Common.Interfaces;
using KelownaSoftware.Application.Common.Notifications.Email;
using KelownaSoftware.Application.Common.Utilities;
using KelownaSoftware.Domain.AggregateRoots.Users;
using KelownaSoftware.Platform.Common.Extensions;
using MediatR;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Identity;

namespace KelownaSoftware.Application.AggregateRoots.Authentication.ResetPassword
{
    public class ResetPasswordCommand : IRequest<ResetPasswordResult>
    {
        public string EncodedEmail { get; set; }
        public string Token { get; set; }
        public string Password { get; set; }

        public class ResetPasswordCommandHandler : IRequestHandler<ResetPasswordCommand, ResetPasswordResult>
        {
            private readonly UserManager<User> _userManager;
            private readonly IEmailService _emailService;
            private readonly IHttpContextAccessor _httpContextAccessor;

            public ResetPasswordCommandHandler(UserManager<User> userManager, IEmailService emailService, IHttpContextAccessor httpContextAccessor)
            {
                _userManager = userManager;
                _emailService = emailService;
                _httpContextAccessor = httpContextAccessor;
            }

            public async Task<ResetPasswordResult> Handle(ResetPasswordCommand request, CancellationToken cancellationToken)
            {
                ResetPasswordResult result;
                var decodedEmail = request.EncodedEmail.FromBase64String();
                var user = await GetUserAsync(decodedEmail);

                //normally we would check if the user exists and throw an exception if null, but in this case,
                //we want to simply return a Success = false and add an Error string
                if (user != null)
                {
                    var decodedToken = Uri.UnescapeDataString(request.Token);
                    var identityResult = await ResetPasswordAsync(user, decodedToken, request.Password, cancellationToken);
                    result = GenerateResultFromIdentityResult(identityResult);

                    if (result.Success)
                    {
                        user.ForcePasswordReset = false;
                        await _userManager.UpdateAsync(user);
                    }
                }
                else
                {
                    result = GenerateResultForUnknownUser();
                }

                return result;

            }

            public string DecodeEmail(string encodedEmail)
            {
                var decodedEmail = encodedEmail.FromBase64String();
                return decodedEmail;

            }

            public async Task<User> GetUserAsync(string email)
            {
                var user = await _userManager.FindByNameAsync(email);
                return user;
            }

            public async Task<IdentityResult> ResetPasswordAsync(User user, string token, string password, CancellationToken cancellationToken)
            {
                var identityResult = await _userManager.ResetPasswordAsync(user, token, password);

                if (identityResult.Succeeded)
                {
                    //Intentional fire and forget
                    SendPasswordHasBeenResetEmailNoficationAsync(user.Email, cancellationToken);
                }

                return identityResult;
            }

            public ResetPasswordResult GenerateResultFromIdentityResult(IdentityResult identityResult)
            {
                var result = new ResetPasswordResult();

                result.Success = identityResult.Succeeded;
                if (identityResult.Succeeded == false)
                {
                    result.Error = identityResult.Errors.FirstOrDefault().Description ?? throw new Exception("An unknown error ocurred");
                }

                return result;
            }

            public ResetPasswordResult GenerateResultForUnknownUser()
            {
                var result = new ResetPasswordResult(false, "The email address specified in the reset password URL cannot be found. Please try resetting your password again.");

                return result;
            }

            public async Task SendPasswordHasBeenResetEmailNoficationAsync(string to, CancellationToken cancellationToken)
            {
                //prevent returning a 500 error, even if an exception occurs while sending the email
                try
                {
                    var baseUrl = HttpContextHelpers.GetBaseUrl(_httpContextAccessor);
                    var passwordHasBeenResetNotification = new PasswordHasBeenResetEmailNotification(to, baseUrl);
                    await _emailService.SendAsync(passwordHasBeenResetNotification, cancellationToken);
                }
                catch
                {
                    //This catch is intentionally empty
                }

            }
        }
    }
}
