﻿using KelownaSoftware.Application.Common.Interfaces.Services;
using KelownaSoftware.Application.Interfaces;
using KelownaSoftware.Application.Interfaces.Authentication;
using KelownaSoftware.Application.Interfaces.Authentication.Jwt;
using KelownaSoftware.Domain.AggregateRoots.Users;
using KelownaSoftware.Platform.Common.Extensions;
using MediatR;
using Microsoft.AspNetCore.Identity;
using Microsoft.EntityFrameworkCore;

namespace KelownaSoftware.Application.AggregateRoots.Authentication.SignIn
{
    public class SignInCommand : IRequest<SignInResult>
    {
        public string Username { get; set; }
        public string Password { get; set; }
        public string? RemoteIP { get; set; }

        public class SignInCommandHandler : IRequestHandler<SignInCommand, SignInResult>
        {
            private readonly UserManager<User> _userManager;
            private readonly IJwtService _jwtService;
            private readonly IRefreshTokenService _refreshTokenService;
            private readonly IAuthenticationService _authenticationService;
            private readonly IKSDbContext _context;
            private readonly IPermissionsService _permissionService;

            public SignInCommandHandler(
                UserManager<User> userManager,
                IJwtService jwtService,
                IRefreshTokenService refreshTokenService,
                IAuthenticationService authenticationService,
                IKSDbContext context,
                IPermissionsService permissionService)
            {
                _userManager = userManager;
                _jwtService = jwtService;
                _refreshTokenService = refreshTokenService;
                _authenticationService = authenticationService;
                _context = context;
                _permissionService = permissionService;
            }

            public async Task<SignInResult> Handle(SignInCommand request, CancellationToken cancellationToken)
            {
                //these are both guaranteed to be not null because of the validator
                var username = request.Username.Trim();
                var password = request.Password.Trim();

                SignInResult result;

                var signInResult = await _authenticationService.CheckUsernamePasswordAreCorrectAsync(username, password);

                if (signInResult.Succeeded)
                {
                    var user = await _context.Users.FirstOrDefaultAsync(x => x.UserName == username, cancellationToken);

                    var claims = await _permissionService.GenerateUserClaimsAsync(user.Id, user.Email, cancellationToken);
                    var jwt = await _jwtService.GenerateEncodedToken(claims);
                    var refreshToken = _refreshTokenService.GenerateToken();

                    await _refreshTokenService.RemoveRefreshTokensAsync(user.Id, cancellationToken);
                    await _refreshTokenService.AddRefreshTokenAsync(refreshToken, user.Id, request.RemoteIP, cancellationToken);

                    if (user.ForcePasswordReset == false)
                    {
                        result = new SignInResult(jwt.Token, refreshToken);
                    }
                    else
                    {
                        var passwordResetToken = await _userManager.GeneratePasswordResetTokenAsync(user);
                        var encodedPasswordResetToken = Uri.EscapeDataString(passwordResetToken);
                        var encodedEmail = username.ToBase64String();
                        result = new SignInResult(jwt.Token, refreshToken, encodedPasswordResetToken, encodedEmail);
                    }
                }
                else
                {
                    result = new SignInResult("Invalid username or password");
                }

                return result;
            }
        }
    }
}
