import { EventEmitter, Injectable, Output } from '@angular/core';
import { Router } from '@angular/router';
import { SignInCommand, SignInResult } from '@application/account';
import { ApiService } from '@infrastructure/services/api.service';
import { LocalStorageConstants } from '@infrastructure/services/services-constants';
import { ReplaySubject, map } from 'rxjs';


@Injectable({
  providedIn: 'root'
})
export class AccountService {

  @Output() userSignedIn: ReplaySubject<void> = new ReplaySubject();
  @Output() userSignedOut: ReplaySubject<void> = new ReplaySubject();

  public get token(): string | null {
    return localStorage.getItem(LocalStorageConstants.token);
  }

  public get refreshToken(): string | null {
    return localStorage.getItem(LocalStorageConstants.refreshToken);
  }

  constructor(
    private apiService: ApiService,
    private router: Router
  ) {
    this.init();
  }

  private init() {
    if (this.token) {
      this.userSignedIn.next();
    }
    else {
      if (this.router.url.startsWith("account/sign-in")) {
        this.signOutAndKeepReturnUrl();
      }
    }
  }


  public signin(command: SignInCommand) {
    return this.apiService.post<SignInResult>(`api/account/signin`, command)
      .pipe(map(result => {
        if (result.error) {
          //do nothing
        }
        else {
          if (result.token) {
            this.storeToken(result.token);
            this.storeRefreshToken(result.refreshToken);
            this.userSignedIn.next();
            
            return result;
          }
          else {
            result.error = "Sign in failed for unknown reason";
          }
        }

        return result;

      }));
  }

  public reauthorize() {
    const token = this.token;
    const refreshToken = this.refreshToken;

    return this.apiService.post<any>(`api/account/refreshToken`, { token, refreshToken })
      .pipe(map(result => {
        this.storeToken(result.token);
        this.storeRefreshToken(result.refreshToken);

        return result;
      }));
  }

  public signOutAndKeepReturnUrl() {
    let returnUrl = this.router.url;
    this.signOut(returnUrl);
  }

  public signOutWithNoReturnUrl() {
    this.signOut(null);
  }

  private signOut(returnUrl: string | null) {
    // remove user from local storage to log user out
    this.userSignedOut.next();
    this.removeTokens();

    if (returnUrl) {
      this.router.navigate(['/account/sign-in'], { queryParams: { returnUrl: returnUrl } });
    }
    else {
      this.router.navigate(['/account/sign-in']);
    }
  }

  private storeToken(token: string) {
    localStorage.setItem(LocalStorageConstants.token, token);
  }

  private storeRefreshToken(refreshToken: string) {
    localStorage.setItem(LocalStorageConstants.refreshToken, refreshToken);
  }

  private removeTokens() {
    localStorage.removeItem(LocalStorageConstants.token);
    localStorage.removeItem(LocalStorageConstants.refreshToken);
  }
}
