import { KeyValue } from '@angular/common';
import { HttpClient, HttpErrorResponse, HttpHeaders, HttpParams, HttpStatusCode } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { BaseResponseViewModel, ValidationErrorViewModel } from '@application/shared';
import { NotificationService } from '@infrastructure/services';
import { Observable, catchError, throwError } from 'rxjs';
import { environment } from 'src/environments/environment';

export class HttpOptions {
  headers?: HttpHeaders | {
    [header: string]: string | string[];
  };
  observe?: 'body';
  params?: HttpParams | {
    [param: string]: string | string[];
  };
  reportProgress?: boolean;
  responseType?: 'json';
  withCredentials?: boolean;
}

export class HttpErrorOptions {
  onValidationErrors: Function;
  ignoreErrors: boolean;

  constructor(onValidationError: Function, ignoreErrors: boolean = false) {
    this.onValidationErrors = onValidationError;
    this.ignoreErrors = ignoreErrors;
  }
}

@Injectable({
  providedIn: 'root'
})
export class ApiService {

  constructor(
    private http: HttpClient,
    private notificationService: NotificationService) { }

  public get<T>(relativeUrl: string, options?: HttpOptions, errorOptions?: HttpErrorOptions): Observable<T> {
    return this.http.get<T>(`${environment.apiUrl}/${relativeUrl}`, options)
      .pipe(
        catchError((e) => {
          this.handleHttpError(e, "Failed to GET: " + relativeUrl, errorOptions);
          return throwError(() => new HttpErrorResponse(e));
        })
      );
  }

  public post<T>(relativeUrl: string, body: any | null, options?: HttpOptions, errorOptions?: HttpErrorOptions): Observable<T> {
    if (!options) {
      options = this.createOptions();
    }

    return this.http.post<T>(`${environment.apiUrl}/${relativeUrl}`, body, options)
      .pipe(
        catchError((e) => {
          this.handleHttpError(e, "Failed to POST: " + relativeUrl, errorOptions);
          return throwError(() => new HttpErrorResponse(e));
        })
      );
  }

  public put<T>(relativeUrl: string, body: any | null, options?: HttpOptions, errorOptions?: HttpErrorOptions): Observable<T> {
    if (!options) {
      options = this.createOptions();
    }

    return this.http.put<T>(`${environment.apiUrl}/${relativeUrl}`, body, options)
      .pipe(
        catchError((e) => {
          this.handleHttpError(e, "Failed to PUT: " + relativeUrl, errorOptions);
          return throwError(() => new HttpErrorResponse(e));
        })
      );
  }

  public delete<T>(relativeUrl: string, options?: HttpOptions, errorOptions?: HttpErrorOptions): Observable<T> {
    if (!options) {
      options = this.createOptions();
    }

    return this.http.delete<T>(`${environment.apiUrl}/${relativeUrl}`, options)
      .pipe(
        catchError((e) => {
          this.handleHttpError(e, "Failed to DELETE: " + relativeUrl, errorOptions);
          return throwError(() => new HttpErrorResponse(e));
        })
      );
  }

  public createOptions(params?: Array<KeyValue<string, string>>): HttpOptions {
    const options = {
      headers: new HttpHeaders({
        'Content-Type': 'application/json'
      }),
      params: new HttpParams()
    };

    if (params) {
      for (const param of params) {
        if (param.value !== null) {
          options.params = options.params.set(param.key, param.value);
        }
      }
    }

    return options;
  }

  public createOptionsFromObject(obj: object): HttpOptions {
    const options = {
      headers: new HttpHeaders({
        'Content-Type': 'application/json'
      }),
      params: new HttpParams()
    };

    if (obj) {
      const entries = Object.entries(obj);

      for (const [key, value] of entries) {
        if (value !== null) {
          if (value instanceof Array) {
            for (const val of value) {
              options.params = options.params.append(key, val);
            }
          }
          else {
            options.params = options.params.set(key, value);
          }
        }

      }
    }

    return options;
  }

  public handleHttpError(e: any, title: string, errorOptions?: HttpErrorOptions): void {
    if (!errorOptions?.ignoreErrors) {
      let errorHandled = false;

      if (e instanceof HttpErrorResponse) {
        switch (e.status) {
          case HttpStatusCode.Unauthorized:
            this.notificationService.ShowError("You do not have permission to perform this action", "Unauthorized");
            errorHandled = true;
            break;

          case HttpStatusCode.BadRequest:
            const response = e.error as BaseResponseViewModel;
            if (response?.validationErrors) {
              // a bad request due to validation errors
              if (errorOptions?.onValidationErrors) {
                errorOptions.onValidationErrors(e);
                errorHandled = true;
              }
            }
            break;
        }
      }

      if (!errorHandled) {
        this.showErrorDetails(e);
        this.notificationService.ShowError(e.error, title);
      }
    }
  }

  private showErrorDetails(e: HttpErrorResponse): void {
    if (!environment.production && e && e.error) {
      if (e.error?.validationMessages) {
        const msgs = e.error.validationErrors.map((x: ValidationErrorViewModel) => ValidationErrorViewModel.ToString(x)).join('\r\n');
        console.error('API Request Validation Errors: ' + msgs);
      } else if (e.error.error) {
        console.error('API Request Error: ' + e.error.error);
      }
    }
  }


}


