﻿using KelownaSoftware.Application.Common.Interfaces;
using KelownaSoftware.Application.Common.Notifications.Email;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using SendGrid;
using SendGrid.Helpers.Mail;

namespace KelownaSoftware.Infrastructure.Common.Services
{
    public class SendGridEmailService : IEmailService
    {
        private readonly string _sendGridApiKey;
        private readonly ILogger _logger;

        public SendGridEmailService(IConfiguration config, ILogger logger)
        {
            _sendGridApiKey = config.GetSection("SENDGRID_API_KEY")?.Value;
            _logger = logger;
        }

        public async Task<bool> SendAsync(IEmailNotification email, CancellationToken cancellationToken)
        {
            bool success = false;

            //the purpose of this try catch is to prevent the failure of an email notification
            //causing the API to return an error response
            try
            {
                if (email != null)
                {
                    var msg = new SendGridMessage()
                    {
                        From = new EmailAddress(EmailNotificationConstants.FROMEMAILADDRESS, EmailNotificationConstants.FROMNAME),
                        Subject = email.Subject,
                        PlainTextContent = email.PlainTextContent,
                        HtmlContent = email.HtmlContent,
                    };

                    if (email.To.Any())
                    {
                        foreach (var to in email.To)
                        {
                            msg.AddTo(new EmailAddress(to));
                        }
                    }

                    if (email.CC.Any())
                    {
                        foreach (var cc in email.CC)
                        {
                            msg.AddCc(new EmailAddress(cc));
                        }
                    }

                    if (email.BCC.Any())
                    {
                        foreach (var bcc in email.BCC)
                        {
                            msg.AddBcc(new EmailAddress(bcc));
                        }
                    }

                    if (email.Attachments.Any())
                    {
                        foreach (var attachment in email.Attachments)
                        {
                            var sendgridAttachment = new Attachment()
                            {
                                Filename = attachment.Filename,
                                Content = attachment.Base64Data
                            };

                            msg.AddAttachment(sendgridAttachment);
                        }
                    }

                    var client = new SendGridClient(_sendGridApiKey);
                    var result = await client.SendEmailAsync(msg, cancellationToken);

                    //in the event the email fails(due to incorrect API, over quota, log it)
                    if (result.IsSuccessStatusCode == false)
                    {
                        var content = await result.Body.ReadAsStringAsync();
                        _logger.LogError($"SendGrid Email Error: {content}");
                    }

                    success = result.IsSuccessStatusCode;
                }


            }
            catch (Exception ex)
            {
                //an exception is thrown if the email address is invalid or some other failure in the SendGridMessage class
                success = false;

                _logger.LogWarning(ex, "SendGridEmailService Exception thrown");
            }

            return success;
        }
    }
}
