﻿using KelownaSoftware.Application.Common.Exceptions;
using KelownaSoftware.Application.Interfaces;
using KelownaSoftware.Application.Interfaces.Authentication;
using KelownaSoftware.Domain.AggregateRoots.Authentication;
using KelownaSoftware.Domain.AggregateRoots.Users;
using Microsoft.EntityFrameworkCore;
using System.Security.Cryptography;

namespace KelownaSoftware.Infrastructure.Services.Authorization
{
    public class RefreshTokenService : IRefreshTokenService
    {
        private readonly IKSDbContext _context;

        public RefreshTokenService(IKSDbContext context)
        {
            _context = context;
        }

        public string GenerateToken(int size = 32)
        {
            var randomNumber = new byte[size];
            using (var rng = RandomNumberGenerator.Create())
            {
                rng.GetBytes(randomNumber);
                return Convert.ToBase64String(randomNumber);
            }
        }

        public async Task AddRefreshTokenAsync(string token, string applicationUserId, string remoteIpAddress, CancellationToken cancellationToken)
        {
            //set user last login
            var user = await _context.Users.FindAsync(applicationUserId);

            if (user == null)
            {
                throw new EntityNotFoundException(nameof(User), applicationUserId);
            }

            user.LastLogin = DateTimeOffset.Now;

            var expiresOn = DateTimeOffset.UtcNow + TimeSpan.FromDays(RefreshTokenConstants.DAYS_TO_EXPIRE);
            RefreshToken refreshToken = new RefreshToken(token, expiresOn, applicationUserId, remoteIpAddress);

            _context.RefreshTokens.Add(refreshToken);

            await _context.SaveChangesAsync(cancellationToken).ConfigureAwait(false);

        }

        public async Task RemoveRefreshTokenAsync(string refreshToken, CancellationToken cancellationToken)
        {
            var token = await _context.RefreshTokens.FirstOrDefaultAsync(x => Equals(x.Token, refreshToken)).ConfigureAwait(false);

            if (token != null)
            {
                _context.RefreshTokens.Remove(token);
                await _context.SaveChangesAsync(cancellationToken).ConfigureAwait(false);
            }
        }

        public async Task<bool> HasValidRefreshToken(string userId, string refreshToken)
        {
            var dbRefreshToken = await _context.RefreshTokens.FirstOrDefaultAsync(x => x.ApplicationUserId == userId && x.Token == refreshToken && !x.Deleted).ConfigureAwait(false);

            if (dbRefreshToken != null && dbRefreshToken.Expires > DateTimeOffset.UtcNow)
            {
                return true;
            }

            return false;

        }

        public async Task RemoveRefreshTokensAsync(string applicationUserId, CancellationToken cancellationToken)
        {
            var dbRefreshTokens = _context.RefreshTokens.Where(x => x.ApplicationUserId == applicationUserId && x.Expires <= DateTimeOffset.Now);

            _context.RefreshTokens.RemoveRange(dbRefreshTokens);
            await _context.SaveChangesAsync(cancellationToken);

        }
    }
}
