﻿using KelownaSoftware.Application.Common.Exceptions;
using KelownaSoftware.Application.Common.ViewModels;
using KelownaSoftware.Domain.Common.Exceptions;
using KelownaSoftware.Domain.Common.Validation;
using KelownaSoftware.Platform.Common.Extensions;
using KelownaSoftware.Platform.FluentPropertyValidator;
using MediatR;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;

namespace KelownaSoftware.Server.Areas.Controllers
{
    [Authorize]
    [ApiController]
    [Route("api/[area]/[controller]")]
    public abstract class BaseController : ControllerBase
    {
        private IMediator _mediator;
        protected IMediator Mediator => _mediator ??= HttpContext.RequestServices.GetService<IMediator>();

        public async Task<IActionResult> ExecuteRequest<T>(T request)
        {
            try
            {
                _ = request ?? throw new ArgumentNullException(nameof(request));

                var result = await Mediator.Send(request);

                if (result is FileBaseResponse)
                {
                    var fileDownloadResult = result as FileBaseResponse;

                    if (fileDownloadResult.ValidationErrors.Any() == false)
                    {
                        var fileContentResult = new FileContentResult(fileDownloadResult.Data, fileDownloadResult.MimeType)
                        {
                            FileDownloadName = fileDownloadResult.Filename
                        };

                        return fileContentResult;
                    }

                    return UnprocessableEntity(result);
                }

                return Ok(result);
            }
            catch (NotAuthorizedException ex)
            {
                return Unauthorized(ex.Message);
            }
            catch (Exception ex) when (ex is ArgumentNullException || ex is BaseException || ex is ArgumentException)
            {
                var result = new BaseResponse();
                result.Success = false;
                result.Error = ex.Message;

                return UnprocessableEntity(result);
            }
            catch (Exception ex) when (ex is FluentValidation.ValidationException)
            {
                List<ValidationError> validationErrors = new List<ValidationError>();
                string rawErrors = ex.Message
                    .Replace("\r\n", string.Empty, StringComparison.OrdinalIgnoreCase)
                    .Replace("Validation failed: ", string.Empty, StringComparison.OrdinalIgnoreCase);

                var listErrors = rawErrors.Split("--");
                foreach (var item in listErrors)
                {
                    var itemString = item.Trim();
                    if (!string.IsNullOrEmpty(itemString))
                    {
                        var itemError = item.Split(":");
                        ValidationError response = new ValidationError()
                        {
                            FieldName = itemError[0].Trim().MakeFirstLetterLowerCase(),
                            ValidationMessage = itemError[1].Trim().Replace("'", string.Empty, StringComparison.OrdinalIgnoreCase)
                        };
                        validationErrors.Add(response);
                    }
                }

                var result = new BaseResponse(ex.Message, validationErrors, false);

                // Fluent validations (i.e. the CQRS level validation, as opposed to Domain level Property validation) should be handled
                // by the FailedValidationResponseFactory in .Net pipeline, not here by an exception handling;
                // therefore this exception here is unexpected, and returns 500 error
                return StatusCode((int)System.Net.HttpStatusCode.InternalServerError, result);
            }
            catch (PropertyValidationException ex)
            {
                var result = new BaseResponse()
                {
                    Success = false
                };

                foreach (var error in ex.Errors)
                {
                    result.AddValidationError(new ValidationError(error.PropertyName, error.ErrorMessage));
                }

                return UnprocessableEntity(result);
            }
            catch (Exception ex)
            {
                var result = new BaseResponse()
                {
                    Success = false,
                    Error = ex.Message
                };

                return StatusCode((int)System.Net.HttpStatusCode.InternalServerError, result);
            }
        }

    }
}