using KelownaSoftware.Application;
using KelownaSoftware.Application.Interfaces;
using KelownaSoftware.Infrastructure;
using KelownaSoftware.Infrastructure.Common;
using KelownaSoftware.Persistence;
using KelownaSoftware.Server.Middleware;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.Mvc;
using Microsoft.IdentityModel.Tokens;
using System.Text;
using FluentValidation;
using FluentValidation.AspNetCore;

namespace KelownaSoftware.Server
{
    public class Program
    {
        public static void Main(string[] args)
        {
            var builder = WebApplication.CreateBuilder(args);

            // Add services to the container.
            builder.Services.AddInfrastructureCommon();
            builder.Services.AddInfrastructure();
            builder.Services.AddPersistence(builder.Configuration);
            builder.Services.AddApplication();

            builder.Services.AddHttpContextAccessor();

            builder.Services.AddAuthentication(JwtBearerDefaults.AuthenticationScheme)
            .AddJwtBearer(options =>
            {
                options.TokenValidationParameters = new TokenValidationParameters
                {
                    ValidateIssuer = true,
                    ValidateAudience = true,
                    ValidateLifetime = true,
                    ValidateIssuerSigningKey = true,
                    ValidIssuer = builder.Configuration["Jwt:Issuer"],
                    ValidAudience = builder.Configuration["Jwt:Issuer"],
                    IssuerSigningKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(builder.Configuration["Jwt:PrivateKey"])),
                    ClockSkew = TimeSpan.Zero
                };
            });

            // FluentValidation plugs into .NET pipeline to validate incoming requests and will set the model as invalid if a validation fails;
            // here we define which method is to be called to generate our own http response (and override the default .NET handling)
            builder.Services.Configure<ApiBehaviorOptions>(o =>
            {
                o.InvalidModelStateResponseFactory = FailedValidationResponseFactory.GenerateResponse;
            });

            builder.Services.AddControllers();
            builder.Services.AddValidatorsFromAssemblyContaining<ICurrentUserService>();
            builder.Services.AddFluentValidationAutoValidation();

            var app = builder.Build();

            app.UseDefaultFiles();
            app.UseStaticFiles();

            // Configure the HTTP request pipeline.

            app.UseHttpsRedirection();
            app.UseAuthentication();
            app.UseAuthorization();


            app.MapControllers();

            app.MapFallbackToFile("/index.html");

            app.Run();
        }
    }
}
